@@ -1,11 +1,12 @@
 use super::multivalued::MultiValuedFastFieldWriter;
-use crate::common;
-use crate::fastfield::serializer::FastFieldSerializer;
+use super::serializer::FastFieldStats;
+use super::FastFieldDataAccess;
 use crate::fastfield::{BytesFastFieldWriter, CompositeFastFieldSerializer};
 use crate::indexer::doc_id_mapping::DocIdMapping;
 use crate::postings::UnorderedTermId;
 use crate::schema::{Cardinality, Document, Field, FieldEntry, FieldType, Schema};
 use crate::termdict::TermOrdinal;
+use common;
 use fnv::FnvHashMap;
 use std::collections::HashMap;
 use std::io;
@@ -265,9 +266,9 @@ impl IntFastFieldWriter {
         self.add_val(val);
     }
 
-    /// Extract the stored data
-    pub(crate) fn get_data(&self) -> Vec<u64> {
-        self.vals.iter().collect::<Vec<u64>>()
+    /// get iterator over the data
+    pub(crate) fn iter(&self) -> impl Iterator<Item = u64> + '_ {
+        self.vals.iter()
     }
 
     /// Push the fast fields value to the `FastFieldWriter`.
@@ -281,17 +282,59 @@ impl IntFastFieldWriter {
         } else {
             (self.val_min, self.val_max)
         };
-        let mut single_field_serializer = serializer.new_u64_fast_field(self.field, min, max)?;
-        if let Some(doc_id_map) = doc_id_map {
-            for doc_id in doc_id_map.iter_old_doc_ids() {
-                single_field_serializer.add_val(self.vals.get(doc_id as usize))?;
-            }
-        } else {
-            for val in self.vals.iter() {
-                single_field_serializer.add_val(val)?;
-            }
+        let fastfield_accessor = WriterFastFieldAccessProvider {
+            doc_id_map,
+            vals: &self.vals,
+        };
+        let stats = FastFieldStats {
+            min_value: min,
+            max_value: max,
+            num_vals: self.val_count as u64,
         };
 
-        single_field_serializer.close_field()
+        if let Some(doc_id_map) = doc_id_map {
+            let iter = doc_id_map
+                .iter_old_doc_ids()
+                .map(|doc_id| self.vals.get(doc_id as usize));
+            serializer.create_auto_detect_u64_fast_field(
+                self.field,
+                stats,
+                fastfield_accessor,
+                iter.clone(),
+                iter,
+            )?;
+        } else {
+            serializer.create_auto_detect_u64_fast_field(
+                self.field,
+                stats,
+                fastfield_accessor,
+                self.vals.iter(),
+                self.vals.iter(),
+            )?;
+        };
+        Ok(())
+    }
+}
+
+#[derive(Clone)]
+struct WriterFastFieldAccessProvider<'map, 'bitp> {
+    doc_id_map: Option<&'map DocIdMapping>,
+    vals: &'bitp BlockedBitpacker,
+}
+impl<'map, 'bitp> FastFieldDataAccess for WriterFastFieldAccessProvider<'map, 'bitp> {
+    /// Return the value associated to the given doc.
+    ///
+    /// Whenever possible use the Iterator passed to the fastfield creation instead, for performance reasons.
+    ///
+    /// # Panics
+    ///
+    /// May panic if `doc` is greater than the index.
+    fn get_val(&self, doc: u64) -> u64 {
+        if let Some(doc_id_map) = self.doc_id_map {
+            self.vals
+                .get(doc_id_map.get_old_doc_id(doc as u32) as usize) // consider extra FastFieldReader wrapper for non doc_id_map
+        } else {
+            self.vals.get(doc as usize)
+        }
     }
 }
